# Generated by Django 4.2.5 on 2023-11-06 17:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('reporting_endpoints', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='report',
            name='disposition',
            field=models.CharField(choices=[('enforce', 'Enforce'), ('report', 'Report'), ('unknown', 'Unknown')], max_length=128),
        ),
    ]



from __future__ import annotations

import json

from django.http import HttpRequest
from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST

from reporting_endpoints.models import Report


def trim(value, length=1024):
    if len(value) > 1024:
        return value[: 1024 - 3] + "..."
    return value


@require_POST
@csrf_exempt
def reporting_endpoint(
    request: HttpRequest, endpoint_name: str | None = None
) -> HttpResponse:
    try:
        data = json.loads(request.body.decode())
    except json.JSONDecodeError:
        return HttpResponse("could not parse", status=400)

    if request.headers["content-type"] not in (
        "application/csp-report",
        "application/reports+json",
    ):
        return HttpResponse("incorrect content type", status=400)

    if "csp-report" in data:
        report = data["csp-report"]
        disposition = Report.Disposition.UNKNOWN
        if report.get("disposition") == "enforce":
            disposition = Report.Disposition.ENFORCE
        elif report.get("disposition") == "report":
            disposition = Report.Disposition.REPORT
        Report.objects.create(
            endpoint_name=endpoint_name,
            document_uri=trim(report["document-uri"]),
            referrer=trim(report["referrer"]),
            violated_directive=report["violated-directive"],
            effective_directive=report["effective-directive"],
            original_policy=report["original-policy"],
            disposition=disposition,
            blocked_uri=report["blocked-uri"],
            status_code=report["status-code"],
            sample=trim(report.get("script-sample") or report.get("sample") or ""),
            source_file=report.get("source-file"),
            line_number=report.get("line-number"),
            column_number=report.get("column-number"),
        )
        return HttpResponse("", status=201)

    if not isinstance(data, list):
        data = [data]

    to_save = []
    for report in data:
        to_save.append(
            Report(
                endpoint_name=endpoint_name,
                age=report.get("age"),
                type=report.get("type"),
                url=trim(report.get("url")),
                user_agent=trim(report.get("user_agent")),
                blocked_uri=report["body"]["blockedURL"],
                disposition=report["body"]["disposition"],
                document_uri=trim(report["body"]["documentURL"]),
                effective_directive=report["body"]["effectiveDirective"],
                original_policy=report["body"]["originalPolicy"],
                referrer=trim(report["body"]["referrer"]),
                sample=trim(report["body"]["sample"]),
                status_code=report["body"]["statusCode"],
            )
        )

    Report.objects.bulk_create(to_save)

    return HttpResponse("", status=201)



from __future__ import annotations

import json

import pytest

from reporting_endpoints.models import Report


pytestmark = pytest.mark.django_db


def test_unparseable(client):
    response = client.post(
        "/_/csp-reports/",
        data="YOLO",
        content_type="application/csp-report",
    )

    assert response.status_code == 400
    assert response.content.decode() == "could not parse"


def test_incorrect_content_type(client):
    response = client.post(
        "/_/csp-reports/",
        data="null",
        content_type="application/text",
    )

    assert response.status_code == 400
    assert response.content.decode() == "incorrect content type"


@pytest.mark.parametrize("disposition", ["enforce", "report", "unknown"])
def test_csp_report(client, disposition):
    data = {
        "document-uri": "https://example.com",
        "referrer": "",
        "violated-directive": "script-src-elem",
        "effective-directive": "script-src-elem",
        "original-policy": "script-src 'self'",
        "disposition": disposition,
        "blocked-uri": "inline",
        "status-code": 200,
    }
    response = client.post(
        "/_/csp-reports/",
        data=json.dumps({"csp-report": data}),
        content_type="application/csp-report",
    )

    assert response.status_code == 201

    report = Report.objects.get()

    assert report.age is None
    assert report.type is None
    assert report.url is None
    assert report.user_agent is None
    assert report.document_uri == data["document-uri"]
    assert report.referrer == data["referrer"]
    assert report.violated_directive == data["violated-directive"]
    assert report.effective_directive == data["effective-directive"]
    assert report.original_policy == data["original-policy"]
    assert report.disposition == disposition
    assert report.blocked_uri == data["blocked-uri"]
    assert report.status_code == data["status-code"]
    assert report.sample == ""


def test_csp_report__value_too_long(client):
    data = {
        "document-uri": "https://example.com" + "A" * 1024,
        "referrer": "",
        "violated-directive": "script-src-elem",
        "effective-directive": "script-src-elem",
        "original-policy": "script-src 'self'",
        "disposition": "enforce",
        "blocked-uri": "inline",
        "status-code": 200,
    }
    response = client.post(
        "/_/csp-reports/",
        data=json.dumps({"csp-report": data}),
        content_type="application/csp-report",
    )

    assert response.status_code == 201

    report = Report.objects.get()

    assert report.age is None
    assert report.type is None
    assert report.url is None
    assert report.user_agent is None
    assert report.document_uri == data["document-uri"][:1021] + "..."
    assert report.referrer == data["referrer"]
    assert report.violated_directive == data["violated-directive"]
    assert report.effective_directive == data["effective-directive"]
    assert report.original_policy == data["original-policy"]
    assert report.disposition == data["disposition"]
    assert report.blocked_uri == data["blocked-uri"]
    assert report.status_code == data["status-code"]
    assert report.sample == ""


def test_single_report(client):
    data = {
        "age": 1000,
        "type": "csp-violation",
        "url": "https://example.com/",
        "user_agent": "test-client",
        "body": {
            "documentURL": "https://example.com",
            "referrer": "",
            "violatedDirective": "script-src-elem",
            "effectiveDirective": "script-src-elem",
            "originalPolicy": "script-src 'self'",
            "disposition": "enforce",
            "blockedURL": "inline",
            "statusCode": 200,
            "sample": "piece of code",
        },
    }
    response = client.post(
        "/_/csp-reports/",
        data=json.dumps(data),
        content_type="application/reports+json",
    )

    assert response.status_code == 201

    report = Report.objects.get()

    assert report.age == data["age"]
    assert report.type == data["type"]
    assert report.url == data["url"]
    assert report.user_agent == data["user_agent"]
    assert report.document_uri == data["body"]["documentURL"]
    assert report.referrer == data["body"]["referrer"]
    assert report.violated_directive == ""
    assert report.effective_directive == data["body"]["effectiveDirective"]
    assert report.original_policy == data["body"]["originalPolicy"]
    assert report.disposition == data["body"]["disposition"]
    assert report.blocked_uri == data["body"]["blockedURL"]
    assert report.status_code == data["body"]["statusCode"]
    assert report.sample == data["body"]["sample"]


def test_single_report__value_too_long(client):
    data = {
        "age": 1000,
        "type": "csp-violation",
        "url": "https://example.com/" + "A" * 1024,
        "user_agent": "test-client",
        "body": {
            "documentURL": "https://example.com" + "A" * 1024,
            "referrer": "",
            "violatedDirective": "script-src-elem",
            "effectiveDirective": "script-src-elem",
            "originalPolicy": "script-src 'self'",
            "disposition": "enforce",
            "blockedURL": "inline",
            "statusCode": 200,
            "sample": "piece of code",
        },
    }
    response = client.post(
        "/_/csp-reports/",
        data=json.dumps(data),
        content_type="application/reports+json",
    )

    assert response.status_code == 201

    report = Report.objects.get()

    assert report.age == data["age"]
    assert report.type == data["type"]
    assert report.url == data["url"][:1021] + "..."
    assert report.user_agent == data["user_agent"]
    assert report.document_uri == data["body"]["documentURL"][:1021] + "..."
    assert report.referrer == data["body"]["referrer"]
    assert report.violated_directive == ""
    assert report.effective_directive == data["body"]["effectiveDirective"]
    assert report.original_policy == data["body"]["originalPolicy"]
    assert report.disposition == data["body"]["disposition"]
    assert report.blocked_uri == data["body"]["blockedURL"]
    assert report.status_code == data["body"]["statusCode"]
    assert report.sample == data["body"]["sample"]


def test_multiple_reports(client):
    data = {
        "age": 1000,
        "type": "csp-violation",
        "url": "https://example.com/",
        "user_agent": "test-client",
        "body": {
            "documentURL": "https://example.com",
            "referrer": "",
            "violatedDirective": "script-src-elem",
            "effectiveDirective": "script-src-elem",
            "originalPolicy": "script-src 'self'",
            "disposition": "enforce",
            "blockedURL": "inline",
            "statusCode": 200,
            "sample": "piece of code",
        },
    }
    response = client.post(
        "/_/csp-reports/",
        data=json.dumps([data, data, data]),
        content_type="application/reports+json",
    )

    assert response.status_code == 201
    assert Report.objects.count() == 3

    for report in Report.objects.all():
        assert report.age == data["age"]
        assert report.type == data["type"]
        assert report.url == data["url"]
        assert report.user_agent == data["user_agent"]
        assert report.document_uri == data["body"]["documentURL"]
        assert report.referrer == data["body"]["referrer"]
        assert report.violated_directive == ""
        assert report.effective_directive == data["body"]["effectiveDirective"]
        assert report.original_policy == data["body"]["originalPolicy"]
        assert report.disposition == data["body"]["disposition"]
        assert report.blocked_uri == data["body"]["blockedURL"]
        assert report.status_code == data["body"]["statusCode"]
        assert report.sample == data["body"]["sample"]


def test_multiple_reports__value_too_long(client):
    data = {
        "age": 1000,
        "type": "csp-violation",
        "url": "https://example.com/" + "A" * 1024,
        "user_agent": "test-client",
        "body": {
            "documentURL": "https://example.com" + "A" * 1024,
            "referrer": "",
            "violatedDirective": "script-src-elem",
            "effectiveDirective": "script-src-elem",
            "originalPolicy": "script-src 'self'",
            "disposition": "enforce",
            "blockedURL": "inline",
            "statusCode": 200,
            "sample": "piece of code",
        },
    }
    response = client.post(
        "/_/csp-reports/",
        data=json.dumps([data, data, data]),
        content_type="application/reports+json",
    )

    assert response.status_code == 201
    assert Report.objects.count() == 3

    for report in Report.objects.all():
        assert report.age == data["age"]
        assert report.type == data["type"]
        assert report.url == data["url"][:1021] + "..."
        assert report.user_agent == data["user_agent"]
        assert report.document_uri == data["body"]["documentURL"][:1021] + "..."
        assert report.referrer == data["body"]["referrer"]
        assert report.violated_directive == ""
        assert report.effective_directive == data["body"]["effectiveDirective"]
        assert report.original_policy == data["body"]["originalPolicy"]
        assert report.disposition == data["body"]["disposition"]
        assert report.blocked_uri == data["body"]["blockedURL"]
        assert report.status_code == data["body"]["statusCode"]
        assert report.sample == data["body"]["sample"]



from __future__ import annotations

import json
from typing import Callable

from django.conf import settings
from django.http import HttpRequest
from django.http.response import HttpResponseBase


class ReportingEndpointsMiddleware:
    sync_capable = True
    async_capable = False

    def __init__(
        self,
        get_response: (Callable[[HttpRequest], HttpResponseBase]),
    ) -> None:
        self.get_response = get_response

    def _prepare_reporting_endpoints(self) -> str | None:
        endpoints = getattr(settings, "REPORTING_ENDPOINTS", None)
        if endpoints is None:
            return endpoints
        if isinstance(endpoints, str):
            return endpoints

        endpoint = '{name}="{url}"'.format
        return ",".join(
            [endpoint(name=name, url=url) for name, url in endpoints.items()]
        )

    def _prepare_report_to_endpoints(self) -> str | None:
        endpoints = getattr(settings, "REPORT_TO_ENDPOINTS", None)
        if endpoints is None:
            return endpoints
        if isinstance(endpoints, str):
            return endpoints

        if isinstance(endpoints, dict):
            endpoints = [endpoints]

        return ",".join([json.dumps(endpoint) for endpoint in endpoints])

    def __call__(self, request: HttpRequest) -> HttpResponseBase:
        response = self.get_response(request)

        reporting_endpoints = self._prepare_reporting_endpoints()
        if reporting_endpoints:
            response.headers["Reporting-Endpoints"] = reporting_endpoints
        report_to_endpoints = self._prepare_report_to_endpoints()
        if report_to_endpoints:
            response.headers["Report-To"] = report_to_endpoints

        return response



# Generated by Django 4.2.5 on 2023-09-27 08:11
from __future__ import annotations

from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Report",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                (
                    "endpoint_name",
                    models.CharField(blank=True, max_length=1024, null=True),
                ),
                ("age", models.PositiveIntegerField(blank=True, null=True)),
                ("type", models.CharField(blank=True, max_length=256, null=True)),
                ("url", models.CharField(blank=True, max_length=1024, null=True)),
                (
                    "user_agent",
                    models.CharField(blank=True, max_length=1024, null=True),
                ),
                ("blocked_uri", models.CharField(max_length=1024)),
                (
                    "disposition",
                    models.CharField(
                        choices=[("enforce", "Enforce"), ("report", "Report")],
                        max_length=128,
                    ),
                ),
                ("document_uri", models.CharField(max_length=1024)),
                ("effective_directive", models.CharField(max_length=1024)),
                ("original_policy", models.TextField()),
                ("referrer", models.CharField(max_length=1024)),
                ("status_code", models.PositiveIntegerField()),
                ("violated_directive", models.CharField(max_length=1024)),
                ("sample", models.CharField(max_length=1024)),
                (
                    "source_file",
                    models.CharField(blank=True, max_length=1024, null=True),
                ),
                ("line_number", models.PositiveIntegerField(blank=True, null=True)),
                ("column_number", models.PositiveIntegerField(blank=True, null=True)),
            ],
        ),
    ]



from __future__ import annotations

from django.apps import AppConfig


class ReportingEndpointsConfig(AppConfig):
    name = "reporting_endpoints"
    verbose_name = "Policy Violation Reports"
    default_auto_field = "django.db.models.AutoField"



from __future__ import annotations

import pytest

from reporting_endpoints.models import Report


pytestmark = pytest.mark.django_db


@pytest.fixture
def admin(django_user_model):
    return django_user_model.objects.create_user(
        username="admin",
        password="hunter42",
        is_active=True,
        is_staff=True,
        is_superuser=True,
    )


def test_admin__listview(client, admin):
    client.force_login(admin)

    response = client.get("/admin/reporting_endpoints/")

    assert response.status_code == 200


def test_admin__changeview(client, admin):
    client.force_login(admin)

    report = Report.objects.create(
        endpoint_name="test-suite",
        age=0,
        type="csp-violation",
        url="https://example.com/",
        user_agent="test-client",
        blocked_uri="https://example.org/",
        disposition="report",
        document_uri="https://example.com/",
        effective_directive="script-src-elem",
        original_policy="script-src 'self'",
        referrer="",
        sample="",
        status_code=200,
    )

    response = client.get(f"/admin/reporting_endpoints/report/{report.id}/change/")

    assert response.status_code == 200
    assert str(report) in response.content.decode()



from __future__ import annotations

from django.contrib import admin

from reporting_endpoints.models import Report


@admin.register(Report)
class ReportAdmin(admin.ModelAdmin):  # type: ignore[type-arg]
    readonly_fields = [
        "age",
        "blocked_uri",
        "column_number",
        "created_at",
        "disposition",
        "document_uri",
        "effective_directive",
        "endpoint_name",
        "line_number",
        "original_policy",
        "referrer",
        "type",
        "sample",
        "source_file",
        "status_code",
        "url",
        "user_agent",
        "violated_directive",
    ]
    list_display = [
        "endpoint_name",
        "created_at",
        "disposition",
        "effective_directive",
        "blocked_uri",
    ]
    list_filter = [
        "endpoint_name",
        "disposition",
        "effective_directive",
        "violated_directive",
        "status_code",
        "type",
    ]
    fieldsets = [
        (
            "Delivery Metadata",
            {
                "fields": [
                    "created_at",
                    ("endpoint_name", "type"),
                    "age",
                    "url",
                    "user_agent",
                ]
            },
        ),
        (
            "Report",
            {
                "fields": [
                    "disposition",
                    "blocked_uri",
                    "document_uri",
                    "effective_directive",
                    "original_policy",
                    "referrer",
                    "status_code",
                    "violated_directive",
                ]
            },
        ),
        (
            "Sample",
            {"fields": [("source_file", "line_number", "column_number"), "sample"]},
        ),
    ]



from __future__ import annotations

from django.db import models


class Report(models.Model):
    class Disposition(models.TextChoices):
        ENFORCE = "enforce"
        REPORT = "report"
        UNKNOWN = "unknown"

    created_at = models.DateTimeField(auto_now_add=True)
    endpoint_name = models.CharField(max_length=1024, null=True, blank=True)

    age = models.PositiveIntegerField(null=True, blank=True)
    type = models.CharField(max_length=256, null=True, blank=True)
    url = models.CharField(max_length=1024, null=True, blank=True)
    user_agent = models.CharField(max_length=1024, null=True, blank=True)

    blocked_uri = models.CharField(max_length=1024)
    disposition = models.CharField(max_length=128, choices=Disposition.choices)
    document_uri = models.CharField(max_length=1024)
    effective_directive = models.CharField(max_length=1024)
    original_policy = models.TextField()
    referrer = models.CharField(max_length=1024)
    status_code = models.PositiveIntegerField()
    violated_directive = models.CharField(max_length=1024)
    sample = models.CharField(max_length=1024)
    source_file = models.CharField(max_length=1024, null=True, blank=True)
    line_number = models.PositiveIntegerField(null=True, blank=True)
    column_number = models.PositiveIntegerField(null=True, blank=True)

    def __str__(self) -> str:
        return f"Report(disposition={self.disposition}, id={self.id})"



import pytest

from reporting_endpoints.models import Report

pytestmark = pytest.mark.django_db


def test_str():
    report = Report.objects.create(
        id=1,
        endpoint_name="test-suite",
        age=0,
        type="csp-violation",
        url="https://example.com/",
        user_agent="test-client",
        blocked_uri="https://example.org/",
        disposition="report",
        document_uri="https://example.com/",
        effective_directive="script-src-elem",
        original_policy="script-src 'self'",
        referrer="",
        sample="",
        status_code=200,
    )

    assert str(report) == "Report(disposition=report, id=1)"
